@@ -1,9 +1,9 @@
-package Parse::RecDescent::FAQ;  # -*- auto-fill -*-
+package Parse::RecDescent::FAQ; 
 
 use vars qw($VERSION);
 
 
-our $VERSION = sprintf '%s', q$Revision: 3.25 $ =~ /Revision:\s+(\S+)\s+/ ;
+our $VERSION = sprintf '%s', q$Revision: 3.3 $ =~ /Revision:\s+(\S+)\s+/ ;
 
 1;
 __END__
@@ -16,12 +16,29 @@ Parse::RecDescent::FAQ - the official, authorized FAQ for Parse::RecDescent.
 
 =head2 Is Parse::RecDescent LL(1)? LL(N)? LR(1)? LR(N)?
 
-A very complete answer to that question is available here:
+=over 4
 
-    http://www.perlmonks.org/index.pl?lastnode_id=480&node_id=153155
+=item Answer by Yves Orton:
+
+I have a data structure which is
+
+a hash of entries
+where
+an entry is a list/array of sets
+
+I have also a grammar that can parse the syntax of the text files that
+contain the data I want to fill this structure with. Until here
+everything is ok.
+
+Problem: I cannot figure out how to actually FILL the parsed data into
+the structure. I can only decide if a string is grammatically correct
+or not.
 
 
-Also see  L</Left-recursion> under L</PARSER BEHAVIOR>
+
+=back
+
+Also see the "Left-recursion" section under "PARSER BEHAVIOR"
 
 =head1 DEBUGGING
 
@@ -60,66 +77,6 @@ name of the offending rule).
 
 =head1 PARSER BEHAVIOR
 
-=head2 Using rule ordering to get the desired match
-
-This logically belongs under L</THINGS NOT TO DO>, but I put it here
-for maximum reader exposure.
-
-Plagiarized from http://perlmonks.org/index.pl?node_id=303626 :
-
-I have the following BNF and am trying to get it to work with RecDescent. 
-I'm evaluating the WHERE condition on a simplified SQL statement.
-
-This is the BNF:
-
-	<cond> ::= <x> [ "OR" <cond> ]
-	<x>    ::= <y> [ "AND" <y> ]
-	<y>    ::= "(" <cond> ")" | <z> <op> <z>
-	<z>    ::= [tabname"."]colname | <value>
-	<op>   ::= "<" | ">" | "=" | "<>" | "<=" | ">="
-
-C<tabname> and C<colname> are identifiers which have at most 20 characters, 
-start with a letter, and cannot end in an underscore. 
-C<colname> as seen above can take the forme C<tabname.colname>
-
-This is my grammar as I've converted it:
-
-	condition : x | x /OR/i condition
-	x : y | y /AND/i y
-	y : "(" condition ")" | z OP z 
-	z : TABCOLNAME | COLNAME | VALUE
-    
-	# Terminals
-	TABCOLNAME : /([a-z]{1}[\w]{0,18}[^_])\.([a-z]{1}[\w]{0,18}[^_\ ])/i
-	COLNAME : /([a-z]{1}[\w]{0,18}[^_\ ])/i
-	VALUE : /(\w+)/ix
-	OP : /(<>|<=|>=|=|<|>)/
-if my string is when parsing 
-
-	"ssn = 1234 and name = bob" 
-
-only 
-
-	"ssn = 1234" 
-
-gets put in @item.
-
-And Abigail doth respondeth:
-
-That's because you give Parse::RecDescent options for the x and y
-rules. PRD will, just like Perl regexes, try alternatives in order,
-left one first. So, if PRD has to choose between C<y> and C<y /AND/i y>,
-it'll first do the C<y>, and only if that alternative fails, it'll try
-the alternative. 
-
-You can fix this by requiring that after matching the SQL statement, it should match the end of the string. But, for efficiency, also reorganize the rules:
-
-	condition:  x /OR/i x
-	         |  x
-	x:          y /AND/i y
-	         |  y
-
-
 =head2 Insuring a top-level rule match
 
 I have a question regarding subrules within grammars used with
@@ -141,7 +98,7 @@ not being matched. I am sure this is a relatively straight-forward
 oversight within the grammar on my part, but I am at a loss as to how
 to correct this.  
 
-Program fragment delivered error ``couldnt open file : No such file or directory at ./tt.pl line 18.''
+Program fragment delivered error ``couldnt open file : No such file or directory at ./tt.pl line 18, <F> line 14.''
 
 =head3 Answer by Randal L. Schwartz
 
@@ -294,9 +251,9 @@ which is a literal dot.
 
 =head2 Left-recursion
 
-=head3 Elimination of
+=over 4
 
-On elimination of left-recursion Randal Schwartz states:
+=item * On elimination of left-recursion Randal Schwartz states:
 
 I had a fun time eliminating a mutual left-recursion problem for
 my "Data-Undumper" program, which used P::RD as an essential component.
@@ -304,21 +261,18 @@ See my discussion of such at
 
  http://www.stonehenge.com/merlyn/LinuxMag/col29.html
 
-Also, regarding elimination see this Perlmonks node:
+=item * Also, regarding elimination see this Perlmonks node:
 
  http://www.perlmonks.org/index.pl?lastnode_id=6364&node_id=153155
 
-And finally "The man(1) of descent" by Conway in The Perl Journal
-issue 12 (also in tutorial directory of the distribution) discusses
-left recursion and its workaround.
-
-=head3 Detection by PRD
+=item * Regarding detection of left-recursion, Conway states:
 
 RecDescent does a complete graph traversal looking for n-ary
 left-recursion loops and fails to compile the grammar if it finds any
 left-recursive loop involving any number of rules. It has done this since its
 earliest versions.
 
+=back
 
 
 
@@ -428,7 +382,7 @@ Do something like this:
 
 =head1 NEWLINE PROCESSING
 
-=head2 Parsing Windows Init (.ini) Files
+=head2 Parsing Windows Init Files
 
 I'm trying ot use Parse::RecDescent to parse a configuration file which
 looks like this:
@@ -470,16 +424,6 @@ looks like this:
 Also take a look at the example in section 11.2 in "Data Munging with Perl"
 by Dave Cross.
 
-=item * And the newest answer by the tireless Randal L. Schwartz: 
-
-  http://www.stonehenge.com/merlyn/UnixReview/col40.html
-
-=item * And a CPAN release:
-
-Config::Yacp,
-
-	http://search.cpan.org/~tstanley/Config-Yacp-1.16/
-
 =back
 
 
@@ -964,18 +908,6 @@ Then just define C<QuotedText>, C<Comma>, and you're done!
 
 =head1 OPTIMIZING YOUR GRAMMARS
 
-=head2 Dan Sugalski's Practice and Theory of Optimizing PRD Grammars
-
-This link is excellent. So excellent in fact, that I took the time to mirror
-it to my site to avoid it ever being lost:
-
-L<http://www.urth.org/~metaperl/domains/semantic-elements.com/perl/prd/dans-blog/000235.html>
-
-This link provides an excellent discussion of hits and misses in optimizing a
-grammar whose initial parse time started out at around minutes. After whittling
-it down to 5 minutes, Dan is still not happy and continues with even
-more aggressive optimizations! Well worth the read.
-
 =head2 Eliminate backtracking when possible
 
 Let's take a look at two computationally equivalent grammars:
@@ -1779,6 +1711,53 @@ to this:
 
 =head1 PROGRAMMING TOPICS GERMANE TO Parse::RecDescent USE
 
+=head2 Regular Expressions
+
+=head3 Shortest match instead of longest match
+
+What is the Perl idiom for getting the leftmost shortest match?
+For instance, so:
+
+ $a = "banana";
+ $a =~ /b.*n/;  # but different
+ print $&;
+
+yields C<banan>. How can I get it to yield
+C<ban>?
+
+=over 4
+
+=item * Just use the non-greedy quantifier modifier, C<?> :
+
+    $a =~ /b.*?n/;
+
+=back
+
+=head3 Tutorial on positive and negative lookahead and lookbehind
+regexps:
+
+ http://lists.isb.sdnpk.org/pipermail/comp-list/2002-August/001156.html
+
+
+=head3  Text::Balanced  and Regexp::Common::balanced
+
+Use this instead of writing hairy regular expressions to match certain
+common "balanced" forms of text, such as tags and parenthesized text.
+
+See http://perlmonks.org/index.pl?node_id=208285
+for an example of where it would be better suited for the task than
+Parse::RecDescent.
+
+=head3 Japhy's Book on Regular Expressions
+
+Ok, so he hasn't named it yet. It's available online for free!
+
+  http://japhy.perlmonk.org/book
+
+=head3 "Mastering Regular Expressions" by Jeffrey Freidl
+
+
+You still need to know when to use C</.*/> or C</.+/> or C</[^x]*/>
 
 =head2 Double vs Single-quoted strings
 
@@ -1909,17 +1888,13 @@ a production the same regardless of whether P::RD returns a string or a
 list of string.
 
 =over 4
-=item * Use L<Scalar::Listify|Scalar::Listify> from CPAN.
+=item * Use Scalar::Listify from CPAN.
 
 =back
 
-=head2 Shallow versus Deep Copying 
+=head2 A tutorial on Shallow versus Deep Copying 
 
-=head3 An article by Randal Schwartz
-    
-    http://www.stonehenge.com/merlyn/UnixReview/col30.html
-
-=head3 A tutorial written by Philip Newton <nospam.newton@gmx.li>
+Written by "Philip 'Yes, that's my address' Newton" <nospam.newton@gmx.li>
 
 Start off with an array of (references to) arrays: 
 
@@ -2222,80 +2197,11 @@ is by far the likeliest explanation.
 
 =back
 
-=head2 Data::MultiValuedHash on search.cpan.org
-
-L<Data::MultiValuedHash> allows for
-transparent manipulation of single or multiply-valued Perl hash values.
-If you've done any recdescent-grokking, you know how useful this can be
-when extracting information from a parse.
-
-=head2 Regular Expressions
-
-=head3 Shortest match instead of longest match
-
-What is the Perl idiom for getting the leftmost shortest match?
-For instance, so:
-
- $a = "banana";
- $a =~ /b.*n/;  # but different
- print $&;
-
-yields C<banan>. How can I get it to yield
-C<ban>?
-
-=over 4
-
-=item * Just use the non-greedy quantifier modifier, C<?> :
-
-    $a =~ /b.*?n/;
-
-=back
-
-=head3 Tutorial on positive and negative lookahead and lookbehind
-regexps:
-
- http://lists.isb.sdnpk.org/pipermail/comp-list/2002-August/001156.html
-
-
-=head3  Text::Balanced  and Regexp::Common::balanced
-
-Use this instead of writing hairy regular expressions to match certain
-common "balanced" forms of text, such as tags and parenthesized text.
-
-See http://perlmonks.org/index.pl?node_id=208285
-for an example of where it would be better suited for the task than
-Parse::RecDescent.
-
-=head3 Japhy's Book on Regular Expressions
-
-Ok, so he hasn't named it yet. It's available online for free!
-
-  http://japhy.perlmonk.org/book
-
-=head3 "Mastering Regular Expressions" by Jeffrey Freidl
-
-
-You still need to know when to use C</.*/> or C</.+/> or C</[^x]*/>
 
 
 =head1 RESOURCES
 
-=head2 Practical Parser Examples
-
-=head3 Config::Yacp
-
-Config::Yacp is a module which parses INI-style config
-files using Parse::RecDescent.
-
-=head3 XSH - The XML Editing Shell
-
-XSH:
-
-L<http://xsh.sourceforge.net/>
-
-uses Parse::RecDescent
-
-=head3 Parsing Symbolic Expressions / Boolean Logic
+=head2 Parsing Symbolic Expressions / Boolean Logic
 
 Says Stefan Mueller:
 
@@ -2305,77 +2211,56 @@ You can find a working example grammar for more involved
 part of the L<Math::Symbolic> distribution. 
 It should be fairly easy to modify it to work for boolean logic. 
 
-=head3 MySql to Oracle schema conversion utility
+
+=head2 MySql to Oracle schema conversion utility
 
 Written in Parse::RecDescent by Tim Bunce:
 
-http://groups.google.com/groups?q=recdescent&start=40&hl=en&scoring=d&rnum=41&selm=9km7b1%246vc%241%40FreeBSD.csie.NCTU.edu.tw
-
-=head2 Web Resources
+ http://groups.google.com/groups?q=recdescent&start=40&hl=en&scoring=d&rnum=41&selm=9km7b1%246vc%241%40FreeBSD.csie.NCTU.edu.tw
 
 
-=head3 Hugh Myer's tips on Parse::RecDescent
+=head2 Data::MultiValuedHash on search.cpan.org
 
-http://www.perlmonks.org/index.pl?node_id=180778
+Transparent manipulation of single or multiply-valued Perl hash values.
 
-17 tips you cannot do without.
+=head2 Parse::Recdescent tutorial at www.perl.com
 
-=head3 Craig's Parse::Recdescent Area
+http://www.perl.com/pub/a/2001/06/13/recdecent.html
 
-L<http://www.informatik.uni-trier.de/~smith/perl/recdescent/>
-    
-Contains an article (in English) published during the Zweiter Perl Workshop
-as well as some slides for a presentation:
+=head2 "Safe undumping"
 
-
-=head3 http://www.PerlMonks.org and irc://efnet/perlhelp
-
-Useful sites to get fast help on Perl.
-
-
-=head2 Publications
-
-=head3 "Parse::Recdescent Tutorial", www.perl.com, 6/13/2001, Jeffrey Goff
-
-L<http://www.perl.com/pub/a/2001/06/13/recdecent.html>
-
-=head3 "Parsing Interesting Things", UnixReview 12/2001, Randal Schwartz
-
-L<http://www.stonehenge.com/merlyn/UnixReview/col40.html>
-
-=head3 "Safe Undumping", Linux Magazine, 10/2001, Randal Schwartz
-
-Randal Schwartz uses
+In this Linux Magazine article by Randal Schwartz uses
 Parse::RecDescent to parse Data::Dumper output. Not fast, but quite
 complete. 
 
-L<http://www.stonehenge.com/merlyn/LinuxMag/col29.html>
-
-=head3 "Data Munging with Perl" by Dave Cross
-
-Chapter 11 is focused on parsing with special emphasis on practical use of
-Parse::RecDescent.
-
-=head3 "Object-Oriented Perl" by Damian Conway
-
-This book will aid you in complexity management for large grammars.
+=head2 "Parsing Interesting Things", SysAdminMag.COM, December 2001, Randal Schwartz
 
 
-=head2 Non Parse::RecDescent Perl Parsers
 
-=head3 py
+=head2 py
 
 py, which I assume is short for C<parse yacc>, is a program by
 Mark-Jason Dominus which parses GNU Bison output to produce Perl parsers.
 
 It is obtainable from http://perl.plover.com/py/
 
-=head3 Parse::YAPP
+=head2 Parse::YAPP
 
 A bottom-up parser which will be familiar to those who
 have used Lex and Yacc. Parse::RecDescent is a top-down parser.
 
+=head2 "Data Munging with Perl" by Dave Cross
 
+Chapter 11 is focused on parsing with special emphasis on practical use of
+Parse::RecDescent.
+
+=head2 "Object-Oriented Perl" by Damian Conway
+
+This book will aid you in complexity management for large grammars.
+
+=head2 http://www.PerlMonks.org and irc://efnet/perlhelp
+
+A useful site to get fast help on Perl.
 
 =head1 SOURCES for FAQ MATERIAL
 
@@ -2423,7 +2308,7 @@ The (unwitting) contributors to this FAQ
 
 =item * Matthew Wickline
 
-=item * Randal L. Schwartz
+=item * Randal L. Schwartz (merlyn), Perl hacker
 
 =item * Stefan Mueller
 
